Package: scanMiR
Type: Package
Title: scanMiR
Version: 1.19.0
Depends: R (>= 4.0)
Date: 2025-07-24
Authors@R: c(
    person("Pierre-Luc", "Germain", email="pierre-luc.germain@hest.ethz.ch", 
            role=c("cre","aut"), comment=c(ORCID="0000-0003-3418-4218")),
    person("Michael", "Soutschek", email="michael.soutschek@hest.ethz.ch", role="aut"),
    person("Fridolin", "Gross", email="fridolin.gross@u-bordeaux.fr", role="aut"))
Imports: Biostrings, pwalign, GenomicRanges, IRanges, data.table,
        BiocParallel, methods, Seqinfo, S4Vectors, ggplot2, stats,
        stringi, utils, graphics, grid, seqLogo, cowplot
Suggests: knitr, rmarkdown, BiocStyle, testthat (>= 3.0.0)
Description: 
    A set of tools for working with miRNA affinity models (KdModels),
    efficiently scanning for miRNA binding sites, and predicting target 
    repression. It supports scanning using miRNA seeds, full miRNA sequences 
    (enabling 3' alignment) and KdModels, and includes the prediction of slicing
    and TDMD sites. Finally, it includes utility and plotting functions (e.g. 
    for the visual representation of miRNA-target alignment).
License: GPL-3
VignetteBuilder: knitr
RoxygenNote: 7.3.1
biocViews: miRNA, SequenceMatching, Alignment
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/scanMiR
git_branch: devel
git_last_commit: a482f71
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-30 00:08:09 UTC; biocbuild
Author: Pierre-Luc Germain [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-3418-4218>),
  Michael Soutschek [aut],
  Fridolin Gross [aut]
Maintainer: Pierre-Luc Germain <pierre-luc.germain@hest.ethz.ch>
