# Generated by roxygen2: do not edit by hand

export(KdModelList)
export(aggregateMatches)
export(assignKdType)
export(conservation)
export(dummyKdData)
export(findSeedMatches)
export(get3pAlignment)
export(get8merRange)
export(getKdModel)
export(getKmers)
export(getMatchTypes)
export(getRandomSeq)
export(getSeed8mers)
export(plotKdModel)
export(removeOverlappingRanges)
export(viewTargetAlignment)
exportClasses(KdModel)
exportClasses(KdModelList)
exportMethods("[")
exportMethods(c)
exportMethods(show)
exportMethods(summary)
import(GenomicRanges)
import(ggplot2)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpnworkers)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,RNAString)
importFrom(Biostrings,complement)
importFrom(Biostrings,consensusMatrix)
importFrom(Biostrings,extractAt)
importFrom(Biostrings,padAndClip)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,subseq)
importFrom(Biostrings,xscat)
importFrom(Seqinfo,seqlevels)
importFrom(IRanges,FactorList)
importFrom(IRanges,IRanges)
importFrom(IRanges,RleList)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,head)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(cowplot,plot_grid)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,dcast)
importFrom(data.table,rbindlist)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(methods,is)
importFrom(methods,new)
importFrom(pwalign,nucleotideSubstitutionMatrix)
importFrom(pwalign,pairwiseAlignment)
importFrom(pwalign,pattern)
importFrom(pwalign,subject)
importFrom(pwalign,type)
importFrom(seqLogo,seqLogo)
importFrom(stats,.lm.fit)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stringi,stri_reverse)
importFrom(utils,data)
importFrom(utils,read.delim)
