#' @rdname SampleTranscript
#' @name SampleTranscript
#' @aliases SampleTranscript
#'
#' @title Example transcript sequence
#'
#' @description
#' An artificial transcript sequence used for examples.
#'
#' @return a named character vector of length 1.
NULL

#' @rdname SampleKdModel
#' @name SampleKdModel
#' @aliases SampleKdModel
#'
#' @title Example KdModel (hsa-miR-155-5p)
#'
#' @description
#' `KdModel` for hsa-miR-155-5p, based on Kd predictions from the CNN of
#' [McGeary, Lin et al. (2019)](https://dx.doi.org/10.1126/science.aav1741).
#'
#' @return a `KdModel` object
#' @examples
#' data(SampleKdModel)
#' SampleKdModel
NULL
