% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{SampleTranscript}
\alias{SampleTranscript}
\title{Example transcript sequence}
\value{
a named character vector of length 1.
}
\description{
An artificial transcript sequence used for examples.
}
